@@ -1,4 +1,4 @@
-# Copyright (c) 2026, Thomas Goirand <zigo@debian.org>
+
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -325,6 +325,28 @@ def _test_ssh_connection(server_ip, username, private_key):
         return False
 
 
+# Returns a list, which may be empty.
+def _get_nova_compute_service(conn, host):
+    LOG.debug(f"Fetching nova-compute service for '{host}'.")
+    try:
+        # List all services and filter the one that matches
+        # the given host and binary
+        services = list(conn.compute.services(binary='nova-compute',
+                                              host=host))
+    except TypeError:
+        LOG.debug("Old SDK detected — manually filtering services.")
+        # Older OpenStack SDK do not have binary and host
+        # parameters, so we need to do the filtering by
+        # ourself.
+        all_services = list(conn.compute.services())
+        services = [
+            svc for svc in all_services
+            if svc.binary == 'nova-compute' and svc.host == host
+        ]
+
+    return services
+
+
 def _enable_compute_service(conn, host):
     """Enable the compute service for the given host.
 
@@ -333,17 +355,17 @@ def _enable_compute_service(conn, host):
     """
     try:
         LOG.info(f"Enabling compute service for host '{host}'.")
-        # List all services and filter the one that matches
-        # the given host and binary
-        services = list(conn.compute.services(binary='nova-compute',
-                                              host=host))
+
+        services = _get_nova_compute_service(conn, host)
+
         if not services:
             raise os_exceptions.ResourceNotFound("No nova-compute service "
                                                  f"found for host '{host}'")
 
         for svc in services:
             if svc.status.lower() != 'enabled':
-                conn.compute.enable_service(svc)
+                conn.compute.enable_service(svc, binary='nova-compute',
+                                            host=host)
                 LOG.info(f"Service '{svc.binary}' on host '{host}' "
                          "enabled successfully.")
             else:
@@ -563,10 +585,32 @@ def _remove_trait_from_resource_provider(conn, resource_provider_uuid,
         raise
 
 
+def _find_aggregate(conn, aggregate_name):
+    LOG.info(f"Searching for aggregate '{aggregate_name}'.")
+    try:
+        # Check if aggregate already exists, create it if not.
+        agg = conn.compute.find_aggregate(aggregate_name)
+    except AttributeError:
+        # Fall back to manual filtering for old SDK
+        aggregates = list(conn.compute.aggregates())
+        agg = None
+        for a in aggregates:
+            if a.name == aggregate_name:
+                agg = a
+                break
+
+    if agg is None:
+        LOG.info(f"Aggregate '{aggregate_name}' not found.")
+        return None
+    else:
+        LOG.info(f"Aggregate '{aggregate_name}' exists.")
+        return agg
+
+
 def _remove_compute_from_aggregate(conn, aggregate_name, hostname):
     LOG.info(f"Removing host '{hostname}' from aggregate '{aggregate_name}'.")
 
-    aggregate = conn.compute.find_aggregate(aggregate_name)
+    aggregate = _find_aggregate(conn, aggregate_name)
     if not aggregate:
         LOG.error(f"Aggregate '{aggregate_name}' not found.")
         return
@@ -584,22 +628,19 @@ def _remove_compute_from_aggregate(conn, aggregate_name, hostname):
 
 
 def _check_compute_exists(conn, hostname):
-    # Check if the compute hostname exists.
-    services = list(conn.compute.services(binary='nova-compute',
-                                          host=hostname))
+    LOG.info(f"Checking '{hostname}' exists.")
+    services = _get_nova_compute_service(conn, hostname)
     if not services:
         raise ValueError(f"Compute host '{hostname}' not found in the cluster")
 
 
 def _create_aggregate(conn, aggregate_name):
-    # Check if aggregate already exists, create it if not.
-    agg = conn.compute.find_aggregate(aggregate_name)
+    agg = _find_aggregate(conn, aggregate_name)
     if agg is None:
         LOG.info(f"Creating aggregate '{aggregate_name}'")
         agg = conn.compute.create_aggregate(name=aggregate_name)
     else:
         LOG.info(f"Aggregate '{aggregate_name}' already exists.")
-
     return agg
 
 
@@ -743,19 +784,13 @@ def _volume_set_image_metadata(user_conn, volume,
 
 def _cleanup_aggregate(conn, aggregate_name):
     try:
-        LOG.info(f"Deleting empty aggregate '{aggregate_name}'")
-        aggregate = conn.compute.find_aggregate(aggregate_name)
-        if aggregate:
-            # Check if the aggregate is empty (no hosts)
-            if not aggregate.hosts:
-                conn.compute.delete_aggregate(aggregate)
-                LOG.info(f"Aggregate '{aggregate_name}' deleted successfully.")
-            else:
-                LOG.info(f"Aggregate '{aggregate_name}' still has hosts, "
-                         "not deleting.")
-        else:
+        aggregate = _find_aggregate(conn, aggregate_name)
+        if aggregate is None:
             LOG.info(f"Aggregate '{aggregate_name}' not found, "
-                     "skipping deletion.")
+                     "not deleting.")
+        else:
+            conn.compute.delete_aggregate(aggregate)
+            LOG.info(f"Aggregate '{aggregate_name}' deleted successfully.")
     except Exception as e:
         LOG.warning(f"Could not clean up aggregate '{aggregate_name}': {e}")
 
@@ -813,17 +848,28 @@ def _cleanup_user_resources(conn, project, user, password, volume_name,
         LOG.info(f"Deleting '{project.name}' resources.")
 
         # Create connection with the user's credentials
-        user_conn = openstack.connect(
-            auth_url=conn.session.get_endpoint(
-                service_type='identity'
-            ),
-            username=user.name,
-            password=password,
-            project_name=project.name,
-            project_domain_name='Default',
-            user_domain_name='Default',
-            region_name=getattr(conn.config, 'region_name', None)
-        )
+        if hasattr(openstack, 'connect'):
+            user_conn = openstack.connect(
+                auth_url=conn.session.get_endpoint(
+                    service_type='identity'
+                ),
+                username=user.name,
+                password=password,
+                project_name=project.name,
+                project_domain_name='Default',
+                user_domain_name='Default',
+                region_name=getattr(conn.config, 'region_name', None)
+            )
+        else:
+            user_conn = openstack.connection.Connection(
+                auth_url=conn.session.get_endpoint(service_type='identity'),
+                username=user.name,
+                password=password,
+                project_name=project.name,
+                project_domain_name='Default',
+                user_domain_name='Default',
+                region_name=getattr(conn.config, 'region_name', None)
+            )
 
         # Delete keypair
         LOG.info(f"Deleting keypair '{keypair_name}'")
@@ -895,15 +941,12 @@ def _disable_compute_service(conn, host):
         LOG.info(f"Disabling compute service for host '{host}'.")
         # List all services and filter the one that matches
         # the given host and binary
-        services = list(conn.compute.services(binary='nova-compute',
-                                              host=host))
-        if not services:
-            raise os_exceptions.ResourceNotFound("No nova-compute service "
-                                                 f"found for host '{host}'")
+        services = _get_nova_compute_service(conn, host)
 
         for svc in services:
             if svc.status.lower() != 'disabled':
-                conn.compute.disable_service(svc)
+                conn.compute.disable_service(svc, binary='nova-compute',
+                                             host=host)
                 LOG.info(f"Service '{svc.binary}' on host '{host}' disabled "
                          "successfully.")
             else:
@@ -1016,8 +1059,7 @@ def validate_compute(conn=None, hostname=None,
     _enable_compute_service(conn, hostname)
 
     # Find the AZ of the host, so we ask to spawn the VM on it.
-    services = list(conn.compute.services(binary='nova-compute',
-                                          host=hostname))
+    services = _get_nova_compute_service(conn, hostname)
 
     if not services:
         raise Exception(f"Compute host '{hostname}' not found in the cluster")
@@ -1038,29 +1080,44 @@ def validate_compute(conn=None, hostname=None,
                                                  project_name=project_name,
                                                  user_name=user_name)
 
+    LOG.info(f"Connecting to OpenStack using the new user '{user_name}'.")
     # Create a new OpenStack connection as non-admin
-    user_conn = openstack.connect(
-        auth_url=conn.session.get_endpoint(
-            service_type='identity'
-        ),
-        username=user.name,
-        password=password,
-        project_name=project.name,
-        project_domain_name='Default',
-        user_domain_name='Default',
-        region_name=getattr(conn.config, 'region_name', None)
-    )
+    if hasattr(openstack, 'connect'):
+        user_conn = openstack.connect(
+            auth_url=conn.session.get_endpoint(
+                service_type='identity'
+            ),
+            username=user.name,
+            password=password,
+            project_name=project.name,
+            project_domain_name='Default',
+            user_domain_name='Default',
+            region_name=getattr(conn.config, 'region_name', None)
+        )
+    else:
+        user_conn = openstack.connection.Connection(
+            auth_url=conn.session.get_endpoint(service_type='identity'),
+            username=user.name,
+            password=password,
+            project_name=project.name,
+            project_domain_name='Default',
+            user_domain_name='Default',
+            region_name=getattr(conn.config, 'region_name', None)
+        )
 
     # Find the image to use to create the volume.
+    LOG.info(f"Searching for image '{image_name}'.")
     image = user_conn.image.find_image(image_name)
     if not image:
-        raise Exception("Image 'Debian 13 trixie' not found")
+        raise Exception("Image '{image_name}' not found")
 
     # Create the volume if it doesn't exist.
+    LOG.info(f"Searching for volume '{volume_name}'.")
     volume = user_conn.volume.find_volume(volume_name)
     if volume:
         LOG.info(f"Volume {volume_name} already exists.")
     else:
+        LOG.info(f"Not exists: creating volume '{volume_name}'.")
         volume = user_conn.volume.create_volume(
             name=volume_name,
             size=5,
@@ -1147,5 +1204,14 @@ def validate_compute(conn=None, hostname=None,
     _remove_trait_from_resource_provider(conn, resource_provider_uuid,
                                          current_traits, trait_name)
 
-    LOG.info("Validation and configuration completed: new compute host "
-             f"'{hostname}' is enabled and ready!")
+    if validation_successful is False:
+        LOG.info(f"Compute host '{hostname}' has been disabled after "
+                 "validation failure.")
+    else:
+        if not keep_enabled:
+            LOG.info("Validation and configuration completed: new compute "
+                     f"host '{hostname}' however disabled, because "
+                     "--keey-enabled flag not present. and ready!")
+        else:
+            LOG.info("Validation and configuration completed: new compute "
+                     f"host '{hostname}' is up and ready!")
